/*
  S-Format t@CǗ
  Satofumi KAMIMURA
  $Id$
*/

#include "sformatCtrl.h"
#include <stdio.h>
#include <fstream>
#include <iterator>


SFormat_Ctrl::SFormat_Ctrl(void) : debug_level(0), error_message("no error") {
  char* level = getenv("DEBUG_VIEW");
  if (level) {
    debug_level = atoi(level);
  }
}


SFormat_Ctrl::~SFormat_Ctrl(void) {
}


const char* SFormat_Ctrl::what(void) {
  return error_message.c_str();
}


bool SFormat_Ctrl::load(const char* file) {
  text.clear();

  std::ifstream fin(file);
  text.assign(std::istream_iterator<char>(fin), std::istream_iterator<char>());
  int size = text.size();

  int index = 0;
  srec_t srec;
  while (index < size) {
    int n = parseSFormat(&srec, &text[index]);
    if (n <= 0) {
      error_message = "S-format syntax error.";
      return false;
    }
    if (debug_level > 0) {
      for (int i = 0; i < n; ++i) {
	fprintf(stderr, "%c", text[index + i]);
      }
      fprintf(stderr, "\n");
    }
    index += n;
  }
  return true;
}


int SFormat_Ctrl::hex2int(char ch) {
  return (ch >= '0' && ch <= '9') ? (ch - '0') : (ch - 'A' + 10);
}


int SFormat_Ctrl::parseSFormat(srec_t* srec, const char* line) {
  int index = 0;
  int addr_size = 2;
  int byte_size;
  int i;
  unsigned char check_sum;
  unsigned char data_sum;

  if (line[index++] != 'S') {	// ŏ̕ 'S' ǂ
    return -1;
  }
  srec->type = line[index++] - '0'; // type ̊i[

  // byte_size ̊i[
  byte_size = (hex2int(line[index]) << 4) + hex2int(line[index + 1]);
  index += 2;

  // address ̊i[
  if (srec->type >= 1 && srec->type <= 3) {
    addr_size = srec->type + 1;
  } else if (srec->type >= 7 && srec->type <= 9) {
    addr_size = 11 - srec->type;
  }
  srec->address = 0;
  for (i = 0; i < 2*addr_size; ++i) {
    srec->address = (srec->address << 4) + hex2int(line[index++]);
  }

  // byte_data ̊i[
  srec->data_size = byte_size - addr_size - 1;
  for (i = 0; i < srec->data_size; ++i) {
    srec->byte_data[i] =
      (hex2int(line[index]) << 4) + hex2int(line[index + 1]);
    index += 2;
  }

  // check sum ̎擾ƃ`FbN
  check_sum = (hex2int(line[index]) << 4) + hex2int(line[index + 1]);
  index += 2;
  data_sum = 0;
  for (i = 0; i <= (srec->data_size + addr_size); ++i) {
    data_sum += (hex2int(line[2*i+2]) << 4) + hex2int(line[2*i+3]);
  }
  if (check_sum != (~data_sum & 0xff)) {
    return -1;
  }

  return index;
}
