/*
  SH7045F ւ̏݃vO
  Satofumi KAMIMURA
  $Id$
*/

#include "sformatCtrl.h"
#include "fztatCtrl.h"
#include <stdio.h>
#include <stdlib.h>
#include "wctrl_dat.h"


#ifdef Linux
static char DefaultDevice[] = "/dev/ttyS0";
#else
static char* DefaultDevice = "COM1";
#endif
enum {
  DefaultBaudrate = 115200,
};


static void errorExit(const char* message) {
  printf("Fail: %s\n", message);
  exit(1);
}


static void progressView(int now, int last) {
  if (now == last) {
    printf("\rcomplete! (%d byte)                \n", last);
    return;
  }

  static int pre_percent = -1;
  double percent = 100.0 * now / last;
  if (static_cast<int>(percent) != pre_percent) {
    printf("\r%d / %d (%3.2f %%)", now, last, percent);
    pre_percent = static_cast<int>(percent);
    fflush(stdout);
  }
}


static void printUsage(const char* progname) {
  printf("usage:\n"
	 "\t%s [options] <mot file>\n"
	 "\n"
	 "options:\n"
	 "-h, --help\t\tprint this message.\n"
	 "--port=<PORT>\t\tspecify tty device. (default %s)\n"
	 "--baudrate=<BAUDRATE>\tspecify tty baudrate. (default %d bps)\n",
	 progname, DefaultDevice, DefaultBaudrate);
}


static void printWriteInfo(const char* device,
			   int writeBaudrate, const char* writeMotFile) {
  printf("device: %s\n"
	 "write baudrate: %d\n"
	 "mot file: %s\n", device, writeBaudrate, writeMotFile);
}


int main(int argc, char *argv[]) {
#ifndef Linux
  std::vector<std::string> ports = SerialDevice::getComPorts();
  if (!ports.empty()) {
    DefaultDevice = &ports[0][0];
  }
#endif

  if (argc < 2) {
    printUsage(argv[0]);
    exit(1);
  }

  // IvV擾
  char* writeMotFile = NULL;
  char* device = DefaultDevice;
  int writeBaudrate = DefaultBaudrate;
  for (int i = 1; i < argc; ++i) {
    if (!strcmp("-h", argv[i]) || !strcmp("--help", argv[i])) {
      printUsage(argv[0]);
      exit(0);
    } else if ((strlen(argv[i]) > 7) && !strncmp("--port=", argv[i], 7)) {
      device = &argv[i][7];
    } else if ((strlen(argv[i]) > 11) && !strncmp("--baudrate=", argv[i], 11)){
      writeBaudrate = atoi(&argv[i][11]);
    } else {
      writeMotFile = argv[i];
    }
  }
  if (!writeMotFile) {
    printUsage(argv[0]);
    exit(1);
  }
  printWriteInfo(device, writeBaudrate, writeMotFile);

  // ރt@C̊mF
  printf("check S-format file ... ");
  SFormat_Ctrl mot;
  if (!mot.load(writeMotFile)) {
    errorExit(mot.what());
  }
  printf("O.K.\n");

  // foCXmF
  printf("adjust SH7045F baudrate ... ");
  FZTAT_Ctrl fztat(device, writeBaudrate);
  if (!fztat.adjustBaudrate()) {
    errorExit(fztat.what());
  }
  printf("O.K.\n");

  // ݐvO̓]
  int wctrl_size = sizeof(write_program) -1;
  printf("send write program (%d byte) ... ", wctrl_size);
  fflush(stdout);
  if (!fztat.sendWriteProgram(write_program, wctrl_size)) {
    errorExit(fztat.what());
  }
  printf("O.K.\n");

  // f[^̏
  printf("writing started.\n");
  fztat.setProgressCallback(progressView);
  std::vector<char> code = mot.text;
  if (!fztat.writeRom(&code[0], code.size(), writeBaudrate)) {
    errorExit(fztat.what());
  }

  // I
  printf("exit normaly.\n");
  return 0;
}
